# Copyright (c) 2010, 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Linaro Power Management QA test suite

**URL:** http://git.linaro.org/gitweb?p=tools/pm-qa.git;a=summary

**Default options:** None
"""



from lava_test.core.installers import TestInstaller
from lava_test.core.parsers import TestParser
from lava_test.core.runners import TestRunner
from lava_test.core.tests import Test

DEFAULT_OPTIONS = ""

INSTALLSTEPS = ['git clone git://git.linaro.org/tools/pm-qa.git',
                'cd pm-qa && make clean && make all']
RUNSTEPS = ['cd pm-qa && make check $(OPTIONS)']
DEPS = ['git-core', 'make', 'linux-libc-dev', 'util-linux', 'build-essential']

# test case name is before  ":" , the test log is between ":" and "...",
# the result is after "..."
# Each test case is separated with a test description beginning with "#"

# Example:
####
#### cpufreq_02:
#### test the cpufreq framework is available for governor
####
#cpufreq_02.0/cpu0: checking scaling_available_governors exists...        pass
#cpufreq_02.1/cpu0: checking scaling_governor exists...                   pass
#cpufreq_02.0/cpu1: checking scaling_available_governors exists...        pass
#cpufreq_02.1/cpu1: checking scaling_governor exists...                   pass

PATTERN = (
    "^(?P<test_case_id>[\w/\.]+):"
    "\s+"
    "(?P<message>.+)"
    "\.\.\.\s+"
    "(?P<result>\w+)")


installer = TestInstaller(INSTALLSTEPS, deps=DEPS)
runner = TestRunner(RUNSTEPS, default_options=DEFAULT_OPTIONS)
parser = TestParser(PATTERN, appendall={'result': 'pass'})

testobj = Test(
    test_id="pwrmgmt",
    installer=installer,
    runner=runner,
    parser=parser)



# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from lava_test.core.installers import TestInstaller
from lava_test.core.parsers import TestParser
from lava_test.core.runners import TestRunner
from lava_test.core.tests import Test

"""
This program is a simple test for testing the audio device on a linux host.
The application, testfreq, will test the audio device by playing a sine wave
at A440 and then sampling the input for that frequency - input is coupled to
output with a cable.

**URL:** http://git.linaro.org/gitweb?p=people/kurt-r-taylor/e2eaudiotest.git;a=tree

**Default options:** None
"""

INSTALLSTEPS = ['git clone git://git.linaro.org/people/kurt-r-taylor/e2eaudiotest.git']
DEPS = ['git-core', 'libasound2-dev', 'libfftw3-dev', 'gcc']
DEFAULT_OPTIONS = ""
RUNSTEPS = ['cd e2eaudiotest; gcc testfreq.c utils_alsa.c -lasound -lfftw3 -o testfreq ; ./e2eaudiotest.sh']
PATTERN = "^(?P<test_case_id>\w+):\W+(?P<result>\w+)\W+(?P<measurement>\d+)\W+sinewave"

e2einst = TestInstaller(INSTALLSTEPS, deps=DEPS)
e2erun = TestRunner(RUNSTEPS,default_options=DEFAULT_OPTIONS)
e2eparser = TestParser(PATTERN,
               appendall={'units':'Hz'})
testobj = Test(test_id="e2eaudiotest", installer=e2einst,
                                  runner=e2erun, parser=e2eparser)



import re
import abrek.testdef

RUNSTEPS = ["glmark2-es2"]

class Glmark2Parser(abrek.testdef.AbrekTestParser):
    def parse(self):
        PAT1 = "^\W+(?P<subtest>.*?)\W+FPS:\W+(?P<measurement>\d+)"
        filename = "testoutput.log"
        pat1 = re.compile(PAT1)
        in_results = False
        cur_test = ""
        with open(filename, 'r') as fd:
            for line in fd.readlines():
                if line.find("Precompilation") != -1:
                    in_results = True
                if in_results == True:
                    match = pat1.search(line)
                    if match:
                        d = match.groupdict()
                        d['test_case_id'] = "%s.%s" % (cur_test, d['subtest'])
                        d.pop('subtest')
                        self.results['test_results'].append(d)
                    else:
                        if line.startswith("==="):
                            in_results = False
                        else:
                            cur_test = line.strip()

        if self.fixupdict:
            self.fixresults(self.fixupdict)
        if self.appendall:
            self.appendtoall(self.appendall)

parse = Glmark2Parser(appendall={'units':'fps', 'result':'pass'})
inst = abrek.testdef.AbrekTestInstaller(deps=["glmark2-es2"])
run = abrek.testdef.AbrekTestRunner(RUNSTEPS)

testobj = abrek.testdef.AbrekTest(testname="glmark2-es2", installer=inst,
                                  runner=run, parser=parse)



# Copyright (c) 2010, 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import contextlib
import hashlib
import logging
import os
import shutil
import urllib2
import urlparse

_fake_files = None
_fake_paths = None
_fake_machine = None


def geturl(url, path=""):
    urlpath = urlparse.urlsplit(url).path
    filename = os.path.basename(urlpath)
    if path:
        filename = os.path.join(path, filename)
    fd = open(filename, "w")
    try:
        response = urllib2.urlopen(urllib2.quote(url, safe=":/"))
        fd = open(filename, 'wb')
        shutil.copyfileobj(response, fd, 0x10000)
        fd.close()
        response.close()
    except:
        raise RuntimeError("Could not retrieve %s" % url)
    return filename


def write_file(data, path):
    with open(path, "w") as fd:
        fd.write(data)


def read_file(path):
    global _fake_files
    global _fake_paths
    if _fake_files is not None:
        if path in _fake_files:
            return _fake_files[path]
    if _fake_paths is not None:
        if path in _fake_paths:
            path = _fake_paths[path]
    with open(path, 'rb') as stream:
        return stream.read()


def fake_file(path, data=None, newpath=None):
    """
    Set up a fake file to be read with read_file() in testing
    If data is specified, the string passed as data will be returned instead
    if newpath is specified, the file attempted to be read will be replaced
    by newfile
    """
    global _fake_files
    global _fake_paths
    if data is not None:
        if _fake_files is None:
            _fake_files = {}
        _fake_files[path] = data
    if newpath is not None:
        if _fake_paths is None:
            _fake_paths = {}
        _fake_paths[path] = newpath


def fake_machine(type):
    """
    Set up a fake machine type for testing
    """
    global _fake_machine
    _fake_machine = type


def clear_fakes():
    global _fake_files
    global _fake_paths
    _fake_files = {}
    _fake_paths = {}


def clear_fake_machine():
    global _fake_machine
    _fake_machine = None


def get_machine_type():
    """
    Return the machine type
    """
    global _fake_machine
    if _fake_machine is None:
        return os.uname()[-1]
    return _fake_machine


def mkdir_p(dirname):
    if not os.path.exists(dirname):
        os.makedirs(dirname)


@contextlib.contextmanager
def changed_directory(dirname, make_if_needed=True):
    """
    A context manager for running a piece of code in another
    directory. The directory is created if needed (by default, can
    be changed with make_if_needed).
    """
    orig_dir = os.getcwd()
    if make_if_needed:
        mkdir_p(dirname)
    logging.info("Changing directory to %r", dirname)
    os.chdir(dirname)
    try:
        yield
    finally:
        logging.info("Changing directory to %r", orig_dir)
        os.chdir(orig_dir)


def merge_dict(merge_into, merge_from):
    """
    Merge two dictionaries recursively:

        1) Simple values are overwritten with a logging.warning() message
        2) Lists are appended
        3) Dictionaries are merged recursively
    """
    assert isinstance(merge_into, dict)
    assert isinstance(merge_from, dict)
    for key in merge_from.iterkeys():
        if key in merge_into:
            if (isinstance(merge_from[key], dict)
                and isinstance(merge_into[key], dict)):
                merge_dict(merge_into[key], merge_from[key])
            elif (isinstance(merge_from[key], list)
                  and isinstance(merge_into[key], list)):
                merge_into[key].extend(merge_from[key])
            else:
                logging.warning(
                    "Overwriting existing value of %r:"
                    "%r overwritten with %r",
                    key, merge_into[key], merge_from[key])
                merge_into[key] = merge_from[key]
        else:
            merge_into[key] = merge_from[key]


class Cache(object):
    """
    Simple open-cached-URL class
    """

    _instance = None

    def __init__(self):
        home = os.environ.get('HOME', '/')
        basecache = os.environ.get('XDG_CACHE_HOME',
                     os.path.join(home, '.cache'))
        self.cache_dir = os.path.join(basecache, 'lava_test')

    @classmethod
    def get_instance(cls):
        if cls._instance is None:
            cls._instance = cls()
        return cls._instance

    def _open_cached(self, key, mode="r"):
        """
        Acts like open() but the pathname is relative to the
        lava_test-specific cache directory.
        """

        if "w" in mode and not os.path.exists(self.cache_dir):
            os.makedirs(self.cache_dir)
        if os.path.isabs(key):
            raise ValueError("key cannot be an absolute path")

        stream = open(os.path.join(self.cache_dir, key), mode)
        return stream

    def _key_for_url(self, url):
        return hashlib.sha1(url).hexdigest()

    def _refresh_url_cache(self, key, url):
        with contextlib.nested(
            contextlib.closing(urllib2.urlopen(url)),
            self._open_cached(key, "wb")) as (in_stream, out_stream):
            out_stream.write(in_stream.read())

    @contextlib.contextmanager
    def open_cached_url(self, url):
        """
        Like urlopen.open() but the content may be cached.
        """
        # Do not cache local files, this is not what users would expect

        if url.startswith("file://"):
            stream = urllib2.urlopen(url)
        else:
            key = self._key_for_url(url)
            try:
                stream = self._open_cached(key, "rb")
            except IOError:
                self._refresh_url_cache(key, url)
                stream = self._open_cached(key, "rb")
        try:
            yield stream
        finally:
            stream.close()



# Copyright (c) 2010, 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""
Tiobench is a multi-threaded I/O benchmark. It is used to measure
system performance in four basic operations: sequential read,
random read, sequential write, and random write.

**URL:** http://sourceforge.net/projects/tiobench/

**Default options:** --block=4096 --block=8192 --threads=2 --numruns=2
"""

"""
"""
import re

from lava_test.core.installers import TestInstaller
from lava_test.core.parsers import TestParser
from lava_test.core.runners import TestRunner
from lava_test.core.tests import Test


VERSION = "0.3.3"
URL = ("http://prdownloads.sourceforge.net/tiobench/"
       "tiobench-%s.tar.gz") % VERSION
MD5 = "bf485bf820e693c79e6bd2a38702a128"
DEFAULT_OPTIONS = "--block=4096 --block=8192 --threads=2 --numruns=2"
INSTALLSTEPS = [
    'tar -zxvf tiobench-%s.tar.gz' % VERSION,
    'cd tiobench-%s && make' % VERSION]
RUNSTEPS = [(
    "cd tiobench-%s && "
    "./tiobench.pl $(OPTIONS)") % VERSION]


class TIObenchTestParser(TestParser):

    def parse(self, artifacts):
        # Pattern to match the test case name
        pattern1 = "(?P<test_id>^(Sequential|Random) (Writes|Reads))"
        # Pattern to match the parameter details and measurement
        pattern2 = (".*?(?P<file_size>\d+)\s+(?P<blks_size>\d+)\s+.*?  "
                    "(?P<measurement>((\d|#)+\.?\d*))")
        filename = "testoutput.log"
        pat1 = re.compile(pattern1)
        pat2 = re.compile(pattern2)
        tc_id = None
        with open(filename) as stream:
            for lineno, line in enumerate(stream, 1):
                match1 = pat1.match(line)
                match2 = pat2.search(line)
                if match1:
                    tc_id = match1.group('test_id').replace(" ", "")
                if match2 and tc_id != None:
                    results = match2.groupdict()
                    blks_size = results.pop('blks_size')
                    filesize = results.pop('file_size')
                    results['test_case_id'] = (
                        '%s_%sMBfilesize_%sbytesblksize') % (
                            tc_id, filesize, blks_size)
                    results["log_filename"] = "testoutput.log"
                    results["log_lineno"] = lineno
                    self.results['test_results'].append(
                        self.analyze_test_result(results))

installer = TestInstaller(INSTALLSTEPS, url=URL, md5=MD5)
runner = TestRunner(RUNSTEPS, default_options=DEFAULT_OPTIONS)
parser = TIObenchTestParser(appendall={'units': 'MB/s', 'result': 'pass'})

testobj = Test(
    test_id="tiobench",
    test_version=VERSION,
    installer=installer,
    runner=runner,
    parser=parser)



# Copyright (c) 2010, 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import sys
from subprocess import Popen, PIPE
from lava_test.utils import read_file, get_machine_type


INTEL_KEYMAP = {
    'vendor_id': 'cpu_vendor_name',
    'cpu family': 'cpu_family',
    'model': 'cpu_model',
    'model name': 'cpu_model_name',
    'stepping': 'cpu_stepping',
    'cpu MHz': 'cpu_mhz',
    'flags': 'cpu_features',
}


INTEL_VALMAP = {
    'cpu family': int,
    'model': int,
    'stepping': int,
    'cpu MHz': float,
}


ARM_KEYMAP = {
    'Processor': 'cpu_model_name',
    'Features': 'cpu_features',
    'CPU implementer': 'cpu_implementer',
    'CPU architecture': 'cpu_architecture',
    'CPU variant': 'cpu_variant',
    'CPU part': 'cpu_part',
    'CPU revision': 'cpu_revision',
}


ARM_VALMAP = {
    'CPU implementer': lambda value: int(value, 16),
    'CPU architecture': int,
    'CPU variant': lambda value: int(value, 16),
    'CPU part': lambda value: int(value, 16),
    'CPU revision': int,
}


def _translate_cpuinfo(keymap, valmap, key, value):
    """
    Translate a key and value using keymap and valmap passed in
    """
    newkey = keymap.get(key, key)
    newval = valmap.get(key, lambda x: x)(value)
    return newkey, newval


def get_cpu_devs():
    """
    Return a list of CPU devices
    """
    pattern = re.compile('^(?P<key>.+?)\s*:\s*(?P<value>.*)$')
    cpunum = 0
    devices = []
    cpudevs = []
    cpudevs.append({})
    machine = get_machine_type()
    if machine in ('i686', 'x86_64'):
        keymap, valmap = INTEL_KEYMAP, INTEL_VALMAP
    elif machine.startswith('arm'):
        keymap, valmap = ARM_KEYMAP, ARM_VALMAP

    try:
        cpuinfo = read_file("/proc/cpuinfo")
        for line in cpuinfo.splitlines():
            match = pattern.match(line)
            if match:
                key, value = match.groups()
                key, value = _translate_cpuinfo(keymap, valmap,
                    key, value)
                if cpudevs[cpunum].get(key):
                    cpunum += 1
                    cpudevs.append({})
                cpudevs[cpunum][key] = value
        for c in range(len(cpudevs)):
            device = {}
            device['device_type'] = 'device.cpu'
            device['description'] = 'Processor #{0}'.format(c)
            device['attributes'] = cpudevs[c]
            devices.append(device)
    except IOError:
        print >> sys.stderr, "WARNING: Could not read cpu information"
    return devices


def get_board_devs():
    """
    Return a list of board devices
    """
    devices = []
    attributes = {}
    device = {}
    machine = get_machine_type()
    if machine in ('i686', 'x86_64'):
        try:
            description = read_file('/sys/class/dmi/id/board_name') or None
            vendor = read_file('/sys/class/dmi/id/board_vendor') or None
            version = read_file('/sys/class/dmi/id/board_version') or None
            if description:
                device['description'] = description.strip()
            if vendor:
                attributes['vendor'] = vendor.strip()
            if version:
                attributes['version'] = version.strip()
        except IOError:
            print >> sys.stderr, "WARNING: Could not read board information"
            return devices
    elif machine.startswith('arm'):
        try:
            cpuinfo = read_file("/proc/cpuinfo")
            if cpuinfo is None:
                return devices
            pattern = re.compile("^Hardware\s*:\s*(?P<description>.+)$", re.M)
            match = pattern.search(cpuinfo)
            if match is None:
                return devices
            device['description'] = match.group('description')
        except IOError:
            print >> sys.stderr, "WARNING: Could not read board information"
            return devices
    else:
        return devices
    if attributes:
        device['attributes'] = attributes
    device['device_type'] = 'device.board'
    devices.append(device)
    return devices


def get_mem_devs():
    """ Return a list of memory devices

    This returns up to two items, one for physical RAM and another for swap
    """
    pattern = re.compile('^(?P<key>.+?)\s*:\s*(?P<value>.+) kB$', re.M)

    devices = []
    try:
        meminfo = read_file("/proc/meminfo")
        for match in pattern.finditer(meminfo):
            key, value = match.groups()
            if key not in ('MemTotal', 'SwapTotal'):
                continue
            capacity = int(value) << 10  # Kernel reports in 2^10 units
            if capacity == 0:
                continue
            if key == 'MemTotal':
                kind = 'RAM'
            else:
                kind = 'swap'
            description = "{capacity}MiB of {kind}".format(
                capacity=capacity >> 20, kind=kind)
            device = {}
            device['description'] = description
            device['attributes'] = {'capacity': str(capacity), 'kind': kind}
            device['device_type'] = "device.mem"
            devices.append(device)
    except IOError:
        print >> sys.stderr, "WARNING: Could not read memory information"
    return devices


def get_usb_devs():
    """
    Return a list of usb devices
    """
    pattern = re.compile(
              "^Bus \d{3} Device \d{3}: ID (?P<vendor_id>[0-9a-f]{4}):"
              "(?P<product_id>[0-9a-f]{4}) (?P<description>.*)$")
    devices = []
    try:
        for line in Popen('lsusb', stdout=PIPE).communicate()[0].splitlines():
            match = pattern.match(line)
            if match:
                vendor_id, product_id, description = match.groups()
                attributes = {}
                device = {}
                attributes['vendor_id'] = int(vendor_id, 16)
                attributes['product_id'] = int(product_id, 16)
                device['attributes'] = attributes
                device['description'] = description
                device['device_type'] = 'device.usb'
                devices.append(device)
    except OSError:
        print >> sys.stderr, "WARNING: Could not read usb device information, \
unable to run lsusb, please install usbutils package"
    return devices


def get_hardware_context():
    """
    Return a dict with all of the hardware profile information gathered
    """
    hardware_context = {}
    devices = []
    devices.extend(get_cpu_devs())
    devices.extend(get_board_devs())
    devices.extend(get_mem_devs())
    devices.extend(get_usb_devs())
    hardware_context['devices'] = devices
    return hardware_context



# Copyright (c) 2010, 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import datetime

from lava_test.api.delegates import ITestRunner
from lava_test.extcmd import (DisplayDelegate, ExternalCommandWithDelegate)


class TestRunner(ITestRunner):
    """
    Base class for defining an test runner object.

    This class can be used as-is for simple execution with the expectation that
    the run() method will be called from the directory where the test was
    installed. Steps, if used, should handle changing directories from there to
    the directory where the test was extracted if necessary.  This class can
    also be extended for more advanced functionality.

    :ivar steps:
        list of shell commands to execute
    """
    def __init__(self, steps=None, default_options=None):
        self.steps = steps or []
        self.default_options = default_options

    def __repr__(self):
        return "<%s steps=%r>" % (self.__class__.__name__, self.steps)

    def _run_lava_test_steps(self, artifacts, observer, test_options=None):
        stdout = open(artifacts.stdout_pathname, 'at')
        stderr = open(artifacts.stderr_pathname, 'at')
        delegate = DisplayDelegate(stdout, stderr, observer)
        extcmd = ExternalCommandWithDelegate(delegate)
        run_failed = False
        try:
            for cmd in self.steps:
                # should override the test options in the step ?
                if cmd.find("$(OPTIONS)") > 0:
                    # check if test options is provided or use default options
                    if not test_options and self.default_options is None:
                        raise RuntimeError((
                            "Test options is missing."
                            " No default value was provided."))
                    if not test_options:
                        test_options = self.default_options
                    cmd = cmd.replace("$(OPTIONS)", test_options)

                if observer:
                    observer.about_to_run_shell_command(cmd)
                returncode = extcmd.call(cmd, shell=True)
                if observer:
                    observer.did_run_shell_command(cmd, returncode)
                if returncode != 0:
                    run_failed = True
        finally:
            stdout.close()
            stderr.close()
        return run_failed

    def run(self, artifacts, observer=None, test_options=None):
        """
        Run the test program by executing steps in sequence.

        .. seealso::

            :meth:`~lava_test.api.delegates.TestRunner.run`
        """
        self.starttime = datetime.datetime.utcnow()
        run_failed = self._run_lava_test_steps(
            artifacts, observer, test_options)
        self.endtime = datetime.datetime.utcnow()
        return run_failed



# Copyright (c) 2010, 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import hashlib
import os

from lava_test.api.delegates import ITestInstaller
from lava_test.extcmd import ExternalCommandWithDelegate
from lava_test.utils import geturl


class TestInstaller(ITestInstaller):
    """
    Base class for defining an installer object.

    This class can be used as-is for simple installers, or extended
    for more advanced functionality.

    :ivar steps:
        List of steps to be executed in a shell

    :ivar deps:
        List of Debian or Ubuntu packages to apt-get install before running the
        steps.

    :ivar url:
        Location from which the test suite should be downloaded.

    :ivar md5:
        The md5sum to check the integrety of the download
    """
    def __init__(self, steps=None, deps=None, url=None, md5=None, **kwargs):
        self.steps = steps or []
        self.deps = deps or []
        self.url = url
        self.md5 = md5

    def __repr__(self):
        return "<%s steps=%r deps=%r url=%r md5=%r>" % (
            self.__class__.__name__,
            self.steps, self.deps, self.url, self.md5)

    def _run_shell_cmd(self, cmd, observer):
        if observer:
            observer.about_to_run_shell_command(cmd)
        extcmd = ExternalCommandWithDelegate(observer)
        returncode = extcmd.check_call(cmd, shell=True)
        if observer:
            observer.did_run_shell_command(cmd, returncode)

    def _installdeps(self, observer):
        if self.deps:
            if observer:
                observer.about_to_install_packages(self.deps)
            # XXX: Possible point of target-specific package installation
            cmd = "sudo apt-get install -y --force-yes " + " ".join(self.deps)
            self._run_shell_cmd(cmd, observer)
            if observer:
                observer.did_install_packages(self.deps)

    def _download(self, observer):
        """
        Download the file specified by the url and check the md5.

        Returns the path and filename if successful, otherwise return None
        """
        if not self.url:
            return
        if observer:
            observer.about_to_download_file(self.url)
        filename = geturl(self.url)
        # If the file does not exist, then the download was not
        # successful
        if not os.path.exists(filename):
            raise RuntimeError(
                "Failed to download %r" % self.url)
        if observer:
            observer.did_download_file(self.url)
        if self.md5:
            checkmd5 = hashlib.md5()
            with open(filename, 'rb') as fd:
                data = fd.read(0x10000)
                while data:
                    checkmd5.update(data)
                    data = fd.read(0x10000)
            if checkmd5.hexdigest() != self.md5:
                raise RuntimeError(
                    "md5sum mismatch of file %r, got %s expected %s" % (
                        filename, checkmd5.hexdigest(), self.md5))
        return filename

    def _runsteps(self, observer):
        for cmd in self.steps:
            self._run_shell_cmd(cmd, observer)

    def install(self, observer=None):
        self._installdeps(observer)
        self._download(observer)
        self._runsteps(observer)



# Copyright (c) 2010, 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import absolute_import

import json
import logging
import os
import shutil

from lava_test.api.core import ITest
from lava_test.core.artifacts import TestArtifacts
from lava_test.core.config import get_config
from lava_test.core.installers import TestInstaller
from lava_test.core.parsers import TestParser, NativeTestParser
from lava_test.core.runners import TestRunner
from lava_test.utils import changed_directory


class Test(ITest):
    """
    Reusable class for defining tests.

    This class uses composition instead of inheritance. You should be able to
    customize the parts you care about by providing delegate objects. This
    class can be used by test definition files to create an object that
    contains the building blocks for installing tests, running them, and
    parsing the results.

    :ivar test_id:
        Name of the test or test suite
    :ivar test_version:
        Version of the test or test suite
    :ivar installer:
        ITestInstaller instance to use
    :ivar runner:
        ITestRunner instance to use
    :ivar parser:
        ITestParser instance to use
    """

    def __init__(self, test_id, test_version=None, installer=None, runner=None,
                 parser=None, default_options=None):
        self._test_id = test_id
        self._test_version = test_version
        # Delegate objects
        self.installer = installer
        self.runner = runner
        self.parser = parser
        self.default_options = default_options

        # Config instance
        self._config = get_config()

    def __repr__(self):
        return ("<%s test_id=%r test_version=%r installer=%r runner=%r"
                " parser=%r>") % (
                    self.__class__.__name__, self.test_id, self.test_version,
                    self.installer, self.runner, self.parser)

    @property
    def test_id(self):
        """
        Return the ID of the test.
        """
        return self._test_id

    @property
    def test_version(self):
        """
        Return the version of the test
        """
        return self._test_version

    @property
    def install_dir(self):
        """
        Pathname of a directory with binary and data files installed by the
        test.

        .. versionadded:: 0.2
        """
        return os.path.join(self._config.installdir, self.test_id)

    @property
    def is_installed(self):
        return os.path.exists(self.install_dir)

    def install(self, observer=None):
        if self.is_installed:
            raise RuntimeError(
                "%s is already installed" % self.test_id)
        if not self.installer:
            raise RuntimeError(
                "no installer defined for '%s'" % self.test_id)
        with changed_directory(self.install_dir):
            try:
                logging.debug(
                    "Invoking %r.install(...)", self.installer)
                self.installer.install(observer)
            except:
                self.uninstall()
                raise

    def uninstall(self):
        logging.debug("Removing test %r", self.test_id)
        if os.path.exists(self.install_dir):
            shutil.rmtree(self.install_dir)

    def run(self, observer=None, test_options=None):
        if not self.runner:
            raise RuntimeError(
                "no test runner defined for '%s'" % self.test_id)
        artifacts = TestArtifacts.allocate(self.test_id, self._config)
        with changed_directory(self.install_dir):
            logging.debug("Invoking %r.run(...)", self.runner)
            run_fail = self.runner.run(artifacts, observer, test_options)

        return artifacts, run_fail

    def parse(self, artifacts):
        if self.parser:
            logging.debug("Invoking %r.parse()", self.parser)
            with changed_directory(artifacts.results_dir, False):
                self.parser.parse(artifacts)
            return self.parser.results


class DeclarativeTest(Test):
    """
    Declaretive ITest implementation.

    Declarative test is like :class:`lava_test.core.tests.Test` but cannot
    contain any python code and is completely encapsulated in a .json file.

    The idea is to write .json files that assemble a Test instance using
    readily-available TestInstaller, TestRunner and TestParser subclasses.
    """

    def __init__(self, about):
        self.about = about
        super(DeclarativeTest, self).__init__(self.about.get('test_id'))
        self.installer = TestInstaller(**self.about.get('install', {}))
        self.runner = TestRunner(**self.about.get('run', {}))
        if self.about.get('parse', {}).get('native', False) is True:
            self.parser = NativeTestParser(self)
        else:
            self.parser = TestParser(**self.about.get('parse', {}))

    @classmethod
    def load_from_stream(cls, stream):
        return cls(json.load(stream))

    def save_to_stream(self, stream):
        json.dumps(self.about, stream, indent="2")



# Copyright (c) 2010, 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""
X11perf is an xlib based benchmark to see how fast an X server can execute various tests

**URL:** http://cgit.freedesktop.org/xorg/app/x11perf/

**Default options:** -repeat 3 -aa10text -aa24text -aatrapezoid300 -aatrap2x300 -copypixwin500 -copypixpix500 -comppixwin500 -shmput500 -shmputxy500 -scroll500
"""



from lava_test.core.installers import TestInstaller
from lava_test.core.parsers import TestParser
from lava_test.core.runners import TestRunner
from lava_test.core.tests import Test


x11perf_options = "-repeat 3"

x11perf_tests = [
    # Antialiased text (using XFT)
    "-aa10text",
    "-aa24text",

    # Antialiased drawing (using XRENDER)
    "-aatrapezoid300",
    "-aatrap2x300",

    # Normal blitting
    "-copypixwin500",
    "-copypixpix500",

    # Composited blitting
    "-comppixwin500",

    # SHM put image
    "-shmput500",
    "-shmputxy500",

    "-scroll500",
]

DEFAULT_OPTIONS = "%s %s" % (x11perf_options,  " ".join(x11perf_tests))
RUNSTEPS = ["x11perf $(OPTIONS)"]
PATTERN = "trep @.*\(\W*(?P<measurement>\d+.\d+)/sec\):\W+(?P<test_case_id>.+)"

installer = TestInstaller(deps=["x11-apps"])
runner = TestRunner(RUNSTEPS, default_options=DEFAULT_OPTIONS)
parser = TestParser(PATTERN, appendall={'units': 'reps/s', 'result': 'pass'})

testobj = Test(
    test_id="x11perf",
    installer=installer,
    runner=runner,
    parser=parser)



# Copyright (c) 2010 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""
GStreamer QA system

**URL:** http://git.collabora.co.uk/?p=user/edward/gst-qa-system;a=summary

**Default options:** None
"""

import os
import simplejson
import gzip
import base64

from lava_test.core.installers import TestInstaller
from lava_test.core.parsers import TestParser
from lava_test.core.runners import TestRunner
from lava_test.core.tests import Test


MAX_TEST_CASE_ID_LEN = 100
MAX_ATTR_KEY_LEN = 32
MAX_ATTR_VAL_LEN = 512

MAX_ATTACHMENT_SIZE = 10 * 1024 * 1024

SETTINGS = "/usr/share/insanity/web/settings.py"

# Re-running failing tests is expensive, as it produces very large log files.
# Only remove --no-reruns if you have a small number of failing tests.
RUNSTEPS = [
    "rm -f testrun.db",
    "gst-media-test --no-reruns -t playbin-test --settings %s" % SETTINGS,
    "gst-media-test --no-reruns -t full-gnlfilesource-scenario " \
    "--settings %s" % SETTINGS,
    "gst-media-test --no-reruns -t simple-encoder-scenario",
    "echo ////////////////////",
    "insanity-dumpresults-json testrun.db --all",
]


class InsanityParser(TestParser):

    def parse(self, artifacts):
        filename = "testoutput.log"
        with open(filename) as stream:
            while not stream.readline().startswith("//////////"):
                pass
            results = simplejson.load(stream)

        self._results = results
        self.fixresults({"pass": "pass", "fail": "fail",
                "skip": "skip", "expected-failure": "pass"})
        self.fixlengths()
        self.attach_logfiles()

    def fixlengths(self):
        for t in self.results['test_results']:
            if "test_case_id" in t:
                if len(t["test_case_id"]) > MAX_TEST_CASE_ID_LEN:
                    t["test_case_id"] = \
                            t["test_case_id"][-MAX_TEST_CASE_ID_LEN:]
            if "attributes" in t:
                attributes = t["attributes"]
                for k, v in attributes.items():
                    if len(k) > MAX_ATTR_KEY_LEN:
                        attributes.pop(k)
                        # start includes namespace info
                        k = k[:MAX_ATTR_KEY_LEN]
                        attributes[k] = v
                    if len(v) > MAX_ATTR_VAL_LEN:
                        # end tends to be more useful than the start.
                        attributes[k] = v[-MAX_ATTR_VAL_LEN:]

    def attach_logfiles(self):
        # FIXME: this should use artifacts.attach_file()
        attachments = []
        mime_type = "text/plain"
        total_attachment_size = 0

        for test in self.results["test_results"]:
            pathname = test.get("log_filename", "")
            if not pathname:
                continue
            if not os.path.exists(pathname):
                print "%r not found: skipping." % pathname
                continue

            if pathname.endswith(".gz"):
                stream = gzip.open(pathname, 'rb')
            else:
                stream = open(pathname)

            output_text = stream.read()
            stream.close()

            total_attachment_size += len(output_text)
            if total_attachment_size > MAX_ATTACHMENT_SIZE:
                break

            attachments.append({
                    "pathname": pathname,
                    "mime_type": mime_type,
                    "content":  base64.standard_b64encode(output_text)
                })

        self.results.setdefault("attachments", []).extend(attachments)

    def fixresults(self, fixupdict):
        """Convert results to a known, standard format

        pass it a dict of keys/values to replace
        For instance:
            {"TPASS":"pass", "TFAIL":"fail"}
        This is really only used for qualitative tests
        """
        for t in self.results['test_results']:
            if "result" in t:
                t['result'] = fixupdict[t['result']]


installer = TestInstaller(
    deps=[
        "insanity-tools",
        "samplemedia-minimal",
        "gstreamer0.10-plugins-base",  # videotestsrc et al
        "gstreamer0.10-plugins-good",  # matroskademux et al
        "gstreamer0.10-plugins-bad",
        "gstreamer0.10-plugins-ugly",  # asfdemux et al
        "gstreamer0.10-ffmpeg",  # ffdec_h264 et al
        "gstreamer0.10-gnonlin",  # gnlfilesource
        "gdb",  # debugging backtraces
    ]
)
runner = TestRunner(RUNSTEPS)
parser = InsanityParser()

testobj = Test(
    test_id="insanity",
    installer=installer,
    runner=runner,
    parser=parser)



# Copyright (c) 2010, 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
The Pybench test Suite is an open source test suite that provides a
standardized way to measure the performance of Python implementations.

**URL:** http://python.org

**Default options:** None
"""


from lava_test.core.installers import TestInstaller
from lava_test.core.parsers import TestParser
from lava_test.core.runners import TestRunner
from lava_test.core.tests import Test

VERSION = 'r27'
URL = "http://svn.python.org/projects/python/tags/%s/Tools/pybench/" % VERSION
DEFAULT_OPTIONS = ""
INSTALLSTEPS = ["svn export %s" % URL]
RUNSTEPS = ['python pybench/pybench.py $(OPTIONS)']
DEPS = ['subversion']


# test case name is first column and measurement is average column
#
#Test                             minimum  average  operation  overhead
#         BuiltinFunctionCalls:     85ms    151ms    0.30us    0.147ms
#         BuiltinMethodLookup:      68ms    113ms    0.11us    0.171ms

PATTERN = (
    "^\s+"
    "(?P<test_case_id>\w+)"
    ":"
    "\s+(\d+)ms"
    "\s+"
    "(?P<measurement>\d+)ms")


installer = TestInstaller(INSTALLSTEPS, deps=DEPS)
runner = TestRunner(RUNSTEPS, default_options=DEFAULT_OPTIONS)
parser = TestParser(PATTERN, appendall={'units': 'ms', 'result': 'pass'})

testobj = Test(
    test_id="pybench",
    installer=installer,
    runner=runner,
    parser=parser)



# Copyright (c) 2010, 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import absolute_import
from lava_test.core.config import get_config

import logging


class TestLoader(object):
    """
    Test loader.

    Encapsulates LAVA Test's knowledge of available tests.

    Test can be loaded by name with
    :meth:`lava_test.core.loader.TestLoader.__getitem__()`. Test can also be
    listed by :meth:`lava_test.core.loader.TestLoader.get_providers()` and then
    iterating over tests returned by each provider.
    """

    def __init__(self, config):
        self._config = config

    def get_providers(self):
        """
        Return a generator of available providers
        """
        import pkg_resources
        for provider_info in self._config.registry.get("providers", []):
            entry_point_name = provider_info.get("entry_point")
            module_name, attrs = entry_point_name.split(':', 1)
            attrs = attrs.split('.')
            try:
                entry_point = pkg_resources.EntryPoint(
                    entry_point_name, module_name, attrs,
                    dist=pkg_resources.get_distribution("lava-test"))
                provider_cls = entry_point.load()
                provider = provider_cls(provider_info.get("config", {}))
                yield provider
            except pkg_resources.DistributionNotFound:
                raise RuntimeError(
                    "lava-test is not properly set up."
                    " Please read the README file")
            except ImportError:
                logging.warning("Couldn't load module: %s", module_name)
                logging.warning(
                    "The configuration may need updating, it is stored in %r",
                    get_config().configdir)

    def __getitem__(self, test_id):
        """
        Lookup a test with the specified test_id
        """
        for provider in self.get_providers():
            try:
                return provider[test_id]
            except KeyError:
                pass
        raise KeyError(test_id)

    def get_test_by_name(self, test_id):
        """
        Lookup a test with the specified name

        .. deprecated:: 0.2
            Use __getitem__ instead
        """
        for provider in self.get_providers():
            try:
                return provider[test_id]
            except KeyError:
                pass
        raise ValueError("No such test %r" % test_id)



# Copyright (c) 2012 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
The lttng test executes lttng-tools test suite and reports the result.

**URL:** https://code.launchpad.net/~linaro-foundations/linaro-ubuntu/lava-test-lttng

**Default options:** None
"""

from lava_test.core.installers import TestInstaller
from lava_test.core.parsers import TestParser
from lava_test.core.runners import TestRunner
from lava_test.core.tests import Test

DEFAULT_OPTIONS = ""

INSTALLSTEPS = [
    "apt-get build-dep lttng-tools --yes",
    "bzr branch lp:~linaro-foundations/linaro-ubuntu/lava-test-lttng"]
DEPS = ["bzr", "linux-headers-$(uname -r)", "lttng-modules-dkms"]
RUNSTEPS = ["cd lava-test-lttng; sudo bash -x ./run-test.sh"]
PATTERN = "^(?P<test_case_id>[\w:()]+)\s+\-\s+(?P<result>\w+$)"

installer = TestInstaller(INSTALLSTEPS, deps=DEPS)
runner = TestRunner(RUNSTEPS, default_options=DEFAULT_OPTIONS)
parser = TestParser(PATTERN)

testobj = Test(
    test_id="lttng",
    installer=installer,
    runner=runner,
    parser=parser)



# Copyright (c) 2010, 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
This script automates the automate installation, execution, and
results parsing for the GMPbench test suite.
The GMPbench test Suite is an open source test suite with the goal of
comparing different processors performance against eachother.

**URL:** http://gmplib.org/gmpbench.html

**Default Options:** None

"""


from lava_test.core.installers import TestInstaller
from lava_test.core.parsers import TestParser
from lava_test.core.runners import TestRunner
from lava_test.core.tests import Test


VERSION = '0.2'
URL = "ftp://ftp.gmplib.org/pub/misc/gmpbench-%s.tar.bz2" % VERSION
URL_gexpr = "http://www.gmplib.org/gexpr.c"
DEPS = ['gcc', 'libgmp3-dev', 'wget', 'bzip2']

DEFAULT_OPTIONS = ""
INSTALLSTEPS = ['tar -xjf gmpbench-0.2.tar.bz2',
                'wget -c %s' % URL_gexpr,
                'mv gexpr.c gmpbench-0.2',
                'cd gmpbench-0.2 && gcc -o gexpr gexpr.c  -static -lm']
RUNSTEPS = ['cd  gmpbench-0.2 && PATH=$PATH:. ./runbench $(OPTIONS)']
PATTERN = (
    "\s*(?P<test_case_id>GMPbench\.*\w*\.*\w*):?\s*"
    "(?P<measurement>\d+.\d+)")

installer = TestInstaller(INSTALLSTEPS, deps=DEPS, url=URL)
runner = TestRunner(RUNSTEPS, default_options=DEFAULT_OPTIONS)
parser = TestParser(PATTERN, appendall={
    'units': 'operations/s', 'result': 'pass'})

testobj = Test(
    test_id="gmpbench",
    installer=installer,
    runner=runner,
    parser=parser)



# Copyright (c) 2010, 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
GtkPerf is an application designed to test GTK+ performance using predefined
GTK+ widgets.

**URL:** http://gtkperf.sourceforge.net/

**Default options:** -a -c 500
"""

import re

from lava_test.core.installers import TestInstaller
from lava_test.core.parsers import TestParser
from lava_test.core.runners import TestRunner
from lava_test.core.tests import Test


# Run tests automatically, 500 repetitions each
DEFAULT_OPTIONS = "-a -c 500"

RUNSTEPS = ["LANG=C gtkperf $(OPTIONS)"]


class GtkTestParser(TestParser):

    def parse(self, artifacts):
        PAT1 = (
            "^(?P<test_case_id>\w+)"
            " - (?P<subtest>\w*\W*\w*)"
            " - time:\W+(?P<measurement>\d+\.\d+)")
        PAT2 = (
            "^(?P<test_case_id>\w+)"
            " - time:\W+(?P<measurement>\d+\.\d+)")
        filename = "testoutput.log"
        pat1 = re.compile(PAT1)
        pat2 = re.compile(PAT2)
        with open(filename) as stream:
            for lineno, line in enumerate(stream, 1):
                match = pat1.search(line)
                if match:
                    d = match.groupdict()
                    d['test_case_id'] = "%s.%s" % (
                        d['test_case_id'],
                        d['subtest'])
                    d.pop('subtest')
                    d['test_case_id'] = d['test_case_id'].replace(" ", "_")
                    d["log_filename"] = filename
                    d["log_lineno"] = lineno
                    self.results['test_results'].append(
                        self.analyze_test_result(d))
                else:
                    match = pat2.search(line)
                    if match:
                        d = match.groupdict()
                        d["log_filename"] = filename
                        d["log_lineno"] = lineno
                        self.results['test_results'].append(
                            self.analyze_test_result(d))


installer = TestInstaller(deps=["gtkperf"])
runner = TestRunner(RUNSTEPS, default_options=DEFAULT_OPTIONS)
parser = GtkTestParser(appendall={'units': 'seconds', 'result': 'pass'})

testobj = Test(
    test_id="gtkperf",
    installer=installer,
    runner=runner,
    parser=parser)



# Copyright (c) 2010, 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
This script automates the automate installation, execution, and
results parsing for the Peacekeeper browser benchmark.

**URL:** http://clients.futuremark.com/peacekeeper/index.action

**Default options:** firefox
"""


from lava_test.core.installers import TestInstaller
from lava_test.core.parsers import TestParser
from lava_test.core.runners import TestRunner
from lava_test.core.tests import Test

import os

curdir = os.path.realpath(os.path.dirname(__file__))

DEFAULT_OPTIONS = "firefox"
INSTALLSTEPS = ['cp -rf %s/peacekeeper/* .' % curdir]
RUNSTEPS = ['python peacekeeper_runner.py $(OPTIONS)']
DEPS = ['python-ldtp', 'firefox']

PATTERN = "^(?P<result>\w+): Score = (?P<measurement>\d+)"

installer = TestInstaller(INSTALLSTEPS, deps=DEPS)
runner = TestRunner(RUNSTEPS, default_options=DEFAULT_OPTIONS)
parser = TestParser(PATTERN, appendall={'units': 'point'})

testobj = Test(
    test_id="peacekeeper",
    installer=installer,
    runner=runner,
    parser=parser)



# Copyright (c) 2010, 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
:mod:`lava_test.api.core` -- Interface classes for core LAVA Test features
==========================================================================

.. module: lava_test.api.core

    :synopsis: Interface classes for core LAVA Test features
"""

from abc import abstractmethod, abstractproperty

from lava_test.api import _Interface


class ITest(_Interface):
    """
    Abstract test definition.

    Test definitions allow lava-test to install, remove, run and parse log
    files of automatic tests. While the interface can be implemented directly
    you should use :class:`lava_test.core.tests.Test` that implements the core
    logic and allow you to customize the parts that are needed by providing
    delegates implementing :class:`~lava_test.api.delegates.ITestInstaller`,
    :class:`~lava_test.api.delegates.ITestRunner` and
    :class:`~lava_test.api.delegates.ITestParser`.

    .. seealso:: :ref:`wrapping_existing_test_or_benchmark`
    """

    @abstractproperty
    def is_installed(self):
        """
        True if this test is installed

        .. versionadded:: 0.2
        """

    @abstractproperty
    def test_id(self):
        """
        The unique name of this test
        """

    @abstractmethod
    def install(self, observer):
        """
        Install the test program.

        This creates an install directory under the user's XDG_DATA_HOME
        directory to mark that the test is installed.  The installer's
        install() method is then called from this directory to complete any
        test specific install that may be needed.

        :param observer:
            Observer object that makes it possible to monitor the actions
            performed by the test installer.
        :type observer:
            :class:`~lava_test.api.observers.ITestInstallerObserver`

        .. versionadded:: 0.2
        """

    @abstractmethod
    def uninstall(self):
        """
        Remove the test program

        Recursively remove test specific directory under the user's
        ``XDG_DATA_HOME directory``.  This will both mark the test as removed,
        and clean up any files that were downloaded or installed under that
        directory. Dependencies are intentionally not removed by this.

        .. versionadded:: 0.1
        """

    @abstractmethod
    def run(self, observer, test_options):
        """
        Run the test program and store artifacts.

        :param observer:
            Observer object that makes it possible to monitor the actions
            performed by the test runner.
        :type observer: :class:`~lava_test.api.observers.ITestRunnerObserver`
        :param test_options:
            Arbitrary string that was provided by the user.
        :type options: :class:`str`
        :return: Pair with test run artifacts and exit code
        :rtype:
            :class:`~lava_test.core.artifacts.TestArtifacts` and :class:`bool`

        .. versionadded:: 0.2
        .. versionchanged:: 0.3.1
            Added options argument and changed the return type to be a tuple
        """

    @abstractmethod
    def parse(self, artifacts):
        """
        Parse the artifacts of an earlier run.

        :param artifacts: Object that describes which files should be parsed.
        :type artifacts: :class:`~lava_test.core.artifacts.TestArtifacts`
        :return:
            A dictionary with all the parsed data. In particular this is a
            TestRun part of the dashboard bundle so it should have the
            test_results list of all the results parsed from the artifacts.
        :rtype: :class:`dict`

        .. versionadded:: 0.2
        """


class ITestProvider(_Interface):
    """
    Source of ITest instances.

    Test providers can be used to make lava-test aware of arbitrary collections
    of tests that can be installed and invoked. Internally lava-test uses this
    class to offer built-in tests (via the
    :class:`~lava_test.providers.BuiltInProvider`), out-of-tree tests (via the
    :class:`~lava_test.providers.PkgResourcesProvider`) and declarative tests
    (via the :class:`~lava_test.providers.RegistryProvider`).

    Normally this is not something you would need to implement. If you have a
    large collection of existing tests that can be somehow adapted in bulk, or
    you have your own internal registry of tests that could be adapted this way
    then you might use this interface to simplify test discovery.

    Test providers need to be registered using pkg-resources entry-point
    feature and then added to the lava-test configuration file. See
    :class:`lava_test.config.LavaTestConfig` for details.

    .. versionadded:: 0.2
    """

    @abstractmethod
    def __init__(self, config):
        """
        Initialize test provider with the specified configuration object. The
        configuration object is obtained from the test tool providers registry.
        """

    @abstractmethod
    def __iter__(self):
        """
        Iterates over instances of ITest exposed by this provider
        """

    @abstractmethod
    def __getitem__(self, test_id):
        """
        Return an instance of ITest with the specified id
        """

    @abstractproperty
    def description(self):
        """
        The description string used by `lava-test list-tests`
        """


__all__ = ['ITest', 'ITestProvider']



# Copyright (c) 2012 Linaro
#
# Author: Ricardo Salveti <rsalveti@linaro.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
The Bluetooth Enablement test cases helps validating basic bluetooth
functionality, like if the adapter is available, able to scan, pair with
other device and such.

**URL:** https://code.launchpad.net/~linaro-foundations/linaro-ubuntu/lava-test-bt-enablement

**Default Options:** None
"""

from lava_test.core.installers import TestInstaller
from lava_test.core.parsers import TestParser
from lava_test.core.runners import TestRunner
from lava_test.core.tests import Test

DEFAULT_OPTIONS = ""
INSTALLSTEPS = [(
    "bzr branch lp:~linaro-foundations/linaro-ubuntu/lava-test-bt-enablement"
    " bluetooth-enablement")]
DEPS = ["bzr", "bluez"]
RUNSTEPS = ["cd bluetooth-enablement; sudo bash -x ./run-test.sh"]
PATTERN = "(?P<test_case_id>[a-zA-Z0-9_-]+):\s(?P<result>\w+)"
FIXUPS = {
    "PASS": "pass",
    "FAIL": "fail"
}

installer = TestInstaller(INSTALLSTEPS, deps=DEPS)
runner = TestRunner(RUNSTEPS, default_options=DEFAULT_OPTIONS)
parser = TestParser(PATTERN, fixupdict=FIXUPS)

testobj = Test(
    test_id="bluetooth-enablement",
    installer=installer,
    runner=runner,
    parser=parser)




# Copyright (c) 2010, 2011, 2012  Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
gatortests  - tests for streamline gator driver and daemon
^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

"""

from lava_test.core.installers import TestInstaller
from lava_test.core.parsers import TestParser
from lava_test.core.runners import TestRunner
from lava_test.core.tests import Test


INSTALLSTEPS = ['bzr branch lp:~stubbsdm/+junk/gator-tests']
RUNSTEPS = ["python gator-tests/gatortests.py"]
PATTERN = "^\*\*\*(?P<test_case_id>\w+):\s(?P<result>\w+)\*\*\*"


DEFAULT_OPTIONS = ""

installer = TestInstaller(INSTALLSTEPS, deps=["python"])
runner = TestRunner(RUNSTEPS,default_options=DEFAULT_OPTIONS)
parser = TestParser(pattern=PATTERN)
testobj = Test(test_id="gatortests", installer=installer,runner=runner, parser=parser)





# Copyright (c) 2010, 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from Queue import Queue
import subprocess
import sys
import threading
try:
    import posix
except ImportError:
    posix = None


class ExternalCommand(object):

    def _popen(self, *args, **kwargs):
        if posix:
            kwargs['close_fds'] = True
        return subprocess.Popen(*args, **kwargs)

    def call(self, *args, **kwargs):
        proc = self._popen(*args, **kwargs)
        proc.wait()
        return proc.returncode

    def check_call(self, *args, **kwargs):
        returncode = self.call(*args, **kwargs)
        if returncode != 0:
            raise subprocess.CalledProcessError(
                returncode, kwargs.get("args") or args[0])
        return returncode


class ExternalCommandWithDelegate(ExternalCommand):

    def __init__(self, delegate):
        self._queue = Queue()
        self._delegate = delegate

    def _read_stream(self, stream, stream_name):
        for line in iter(stream.readline, ''):
            cmd = (stream_name, line)
            self._queue.put(cmd)

    def _drain_queue(self):
        while True:
            args = self._queue.get()
            if args is None:
                break
            self._delegate.display_subprocess_output(*args)

    def call(self, *args, **kwargs):
        kwargs['stdout'] = subprocess.PIPE
        kwargs['stderr'] = subprocess.PIPE
        proc = self._popen(*args, **kwargs)
        stdout_reader = threading.Thread(
            target=self._read_stream, args=(proc.stdout, "stdout"))
        stderr_reader = threading.Thread(
            target=self._read_stream, args=(proc.stderr, "stderr"))
        ui_printer = threading.Thread(
            target=self._drain_queue)

        ui_printer.start()
        stdout_reader.start()
        stderr_reader.start()
        try:
            proc.wait()
        except KeyboardInterrupt:
            proc.kill()
        finally:
            stdout_reader.join()
            stderr_reader.join()
            self._queue.put(None)
            ui_printer.join()
        return proc.returncode


class DisplayDelegate(object):
    """
    Delegate for displaying command output.

    Perfect companion for ExternalCommandWithDelegate.
    """

    def __init__(self, stdout=None, stderr=None, chain=None):
        self.stdout = stdout or sys.stdout
        self.stderr = stderr or sys.stderr
        self.chain = chain

    def display_subprocess_output(self, stream_name, line):
        if stream_name == 'stdout':
            self.stdout.write(line)
        elif stream_name == 'stderr':
            self.stderr.write(line)
        if self.chain:
            self.chain.display_subprocess_output(stream_name, line)



# Copyright (c) 2010, 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import decimal
import os
import re

from lava_test.api.delegates import ITestParser


class TestParser(ITestParser):
    """
    Base class for defining a test parser

    This class can be used as-is for simple results parsers, but will likely
    need to be extended slightly for many.  If used as it is, the parse()
    method should be called while already in the results directory and assumes
    that a file for test output will exist called testoutput.log.

    :ivar pattern:
        regexp pattern to identify important elements of test output For
        example: If your testoutput had lines that look like: "test01: PASS"
        then you could use a pattern like this:
        "^(?P<testid>\w+):\W+(?P<result>\w+)" This would result in
        identifying "test01" as testid and "PASS" as result. Once parse()
        has been called, self.results.test_results[] contains a list of
        dicts of all the key,value pairs found for each test result.

    :ivar fixupdict:
        Dict of strings to convert test results to standard strings For
        example: if you want to standardize on having pass/fail results in
        lower case, but your test outputs them in upper case, you could use a
        fixupdict of something like: {'PASS':'pass','FAIL':'fail'}

    :ivar appendall:
        Append a dict to the test_results entry for each result.
        For example: if you would like to add units="MB/s" to each result:
        appendall={'units':'MB/s'}

    :ivar results:
        Dictionary of data that was scrubbed from the log file for this test
        run. Most notably it contains the test_results array.
    """
    def __init__(self, pattern=None, fixupdict=None, appendall={}):
        if pattern is not None:
            try:
                re.compile(pattern)
            except Exception as ex:
                raise ValueError(
                    "Invalid regular expression %r: %s", pattern, ex)
        self._results = {'test_results': []}
        self.pattern = pattern
        self.fixupdict = fixupdict
        self.appendall = appendall

    def __repr__(self):
        return "<%s pattern=%r fixupdict=%r appendall=%r>" % (
            self.__class__.__name__,
            self.pattern, self.fixupdict, self.appendall)

    @property
    def results(self):
        return self._results

    def parse(self, artifacts):
        if os.path.exists(artifacts.stdout_pathname):
            return self.parse_pathname(
                artifacts.stdout_pathname,
                os.path.relpath(artifacts.stdout_pathname,
                                artifacts.results_dir))
        if os.path.exists(artifacts.stderr_pathname):
            return self.parse_pathname(
                artifacts.stderr_pathname,
                os.path.relpath(artifacts.stderr_pathname,
                                artifacts.results_dir))

    def parse_pathname(self, pathname, relative_pathname=None):
        with open(pathname, 'rt') as stream:
            for lineno, line in enumerate(stream, 1):
                match = re.search(self.pattern, line)
                if not match:
                    continue
                data = match.groupdict()
                data["log_filename"] = relative_pathname or pathname
                data["log_lineno"] = lineno
                self._results['test_results'].append(
                    self.analyze_test_result(data))
        return self.results

    @property
    def badchars(self):
        return "[^a-zA-Z0-9\._-]"

    def analyze_test_result(self, data):
        """
        Analyze sigle match (typically single line) and convert it into a
        proper test result object.

        Currently this method does the following transformations:
            * measurement is converted to decimal if present
            * test_case_id is rewritten to strip badchars
            * test_case_id is rewritten to convert spaces to underscores
            * result is transformed using fixuptdict, if defined
            * appendall information is added, if defined
        """
        if 'measurement' in data:
            try:
                data['measurement'] = decimal.Decimal(data['measurement'])
            except decimal.InvalidOperation:
                del data['measurement']
        if 'test_case_id' in data:
            data['test_case_id'] = re.sub(self.badchars, "",
                                          data['test_case_id'])
            data['test_case_id'] = data['test_case_id'].replace(" ", "_")
        if 'result' in data and self.fixupdict:
            data['result'] = self.fixupdict[data['result']]
        if self.appendall:
            data.update(self.appendall)
        return data


class NativeTestParser(ITestParser):
    """
    Unfinished native test parser.

    This was meant to be a pass-through for tests that directly create bundles
    """
    def __init__(self, test_def):
        self.test_def = test_def

    def parse(self, artifacts):
        raise NotImplementedError()

    def results(self):
        raise NotImplementedError()



# Copyright (c) 2010, 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from lava_test.api.core import ITestProvider
from lava_test.core.config import get_config
from lava_test.core.tests import DeclarativeTest
from lava_test.utils import Cache

import logging


class BuiltInProvider(ITestProvider):
    """
    Test provider that provides tests shipped in the Lava-Test source tree
    """

    _builtin_tests = [
        'bluetooth_enablement',
        'bootchart',
        'insanity',
        'e2eaudiotest',
        'firefox',
        'gatortests',
        'glmemperf',
        'gmpbench',
        'gtkperf',
        'leb_basic_graphics',
        'ltp',
        'lttng',
        'peacekeeper',
        'perf',
        'posixtestsuite',
        'pwrmgmt',
        'pybench',
        'smem',
        'stream',
        'tiobench',
        'x11perf',
        'xrestop',
        'wifi_enablement',
    ]

    def __init__(self, config):
        pass

    @property
    def description(self):
        return "Tests built directly into LAVA Test:"

    def __iter__(self):
        return iter(self._builtin_tests)

    def __getitem__(self, test_id):
        if test_id not in self._builtin_tests:
            raise KeyError(test_id)
        module = __import__("lava_test.test_definitions.%s" % test_id,
                            fromlist=[''])
        return module.testobj


class PkgResourcesProvider(ITestProvider):
    """
    Test provider that provides tests declared in pkg_resources working_set

    By default it looks at the 'lava_test.test_definitions' name space but it
    can be changed with custom 'namespace' configuration entry.
    """

    def __init__(self, config):
        self._config = config

    @property
    def namespace(self):
        return self._config.get("namespace", "lava_test.test_definitions")

    @property
    def description(self):
        return ("Tests provided by installed python packages"
                " (from namespace {0}):").format(self.namespace)

    def __iter__(self):
        from pkg_resources import working_set
        for entry_point in working_set.iter_entry_points(self.namespace):
            yield entry_point.name

    def __getitem__(self, test_name):
        from pkg_resources import working_set
        for entry_point in working_set.iter_entry_points(self.namespace,
                                                         test_name):
            return entry_point.load().testobj
        raise KeyError(test_name)


class RegistryProvider(ITestProvider):
    """
    Test provider that provides declarative tests listed in the test registry.
    """
    def __init__(self, config):
        self._config = config
        self._cache = None

    @property
    def entries(self):
        """
        List of URLs to DeclarativeTest description files
        """
        return self._config.get("entries", [])

    @property
    def description(self):
        return "Tests provided by LAVA Test registry:"

    @classmethod
    def register_remote_test(self, test_url):
        config = get_config()  # This is a different config object from
                               # self._config
        provider_config = config.get_provider_config(
            "lava_test.core.providers:RegistryProvider")
        if "entries" not in provider_config:
            provider_config["entries"] = []
        if test_url not in provider_config["entries"]:
            provider_config["entries"].append(test_url)
            config._save_registry()
        else:
            raise ValueError("This test is already registered")

    @classmethod
    def unregister_remote_test(self, test_url):
        # This is a different config object from self._config
        config = get_config()
        provider_config = config.get_provider_config(
            "lava_test.core.providers:RegistryProvider")
        provider_config.get("entries", []).remove(test_url)
        config._save_registry()

    def _load_remote_test(self, test_url):
        """
        Load DeclarativeTest from a (possibly cached copy of) test_url
        """
        cache = Cache.get_instance()
        with cache.open_cached_url(test_url) as stream:
            return DeclarativeTest.load_from_stream(stream)

    def _fill_cache(self):
        """
        Fill the cache of all remote tests
        """
        if self._cache is not None:
            return
        self._cache = {}
        for test_url in self.entries:
            try:
                test = self._load_remote_test(test_url)
                if test.test_id in self._cache:
                    raise ValueError(
                        "Duplicate test %s declared" % test.test_id)
                self._cache[test.test_id] = test
            except IOError as exc:
                logging.warning(
                    "Unable to load test definition from %r: %r",
                    test_url, exc)
                if hasattr(exc, 'reason'):
                    logging.warning("Error reason: %r", exc.reason)
                elif hasattr(exc, 'code'):
                    logging.warning('Error code: %r', exc.code)
            except Exception as exc:
                # This can be a number of things, including URL errors, JSON
                # errors and validation errors
                logging.warning(
                    'Unable to load test definition from %r: %r',
                    test_url, exc)

    def __iter__(self):
        self._fill_cache()
        for test_id in self._cache.iterkeys():
            yield test_id

    def __getitem__(self, test_id):
        self._fill_cache()
        return self._cache[test_id]



# Copyright (c) 2010, 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
import logging.config

from lava_tool.dispatcher import LavaDispatcher, run_with_dispatcher_class


class LAVATestDispatcher(LavaDispatcher):

    toolname = 'lava_test'
    description = """
    LAVA Test wrapper framework
    """
    epilog = """
    Please report all bugs using the Launchpad bug tracker:
    http://bugs.launchpad.net/lava-test/+filebug
    """


def main():
    # default logging level is warning. -v or --verbose will change it to debug
    # (in Command class).
    FORMAT = '<LAVA_TEST>%(asctime)s %(levelname)s: %(message)s'
    DATEFMT = '%Y-%m-%d %I:%M:%S %p'
    logging.basicConfig(format=FORMAT, datefmt=DATEFMT)
    logging.root.setLevel(logging.WARNING)
    return run_with_dispatcher_class(LAVATestDispatcher)


if __name__ == '__main__':
    raise SystemExit(main())



# Copyright (c) 2010, 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


"""
:mod:`lava_test.api.observers` -- Interface classes for observer classes
========================================================================

.. module: lava_test.api.observers
    :synopsis: Interface classes for observer classes
"""

from abc import abstractmethod

from lava_test.api import _Interface


class IShellCommandObserver(_Interface):
    """
    Shell command runner observer class.

    Allows the caller to observe shell commands that occur during some
    operation. It is used by the command line UI.

        .. versionadded:: 0.2
    """

    @abstractmethod
    def about_to_run_shell_command(self, cmd):
        """
        Method called when a shell command is about to be invoked by the
        observed object.

        .. versionadded:: 0.2
        """

    @abstractmethod
    def did_run_shell_command(self, cmd, returncode):
        """
        Method called when a shell command has been invoked by the observed
        object.

        .. versionadded:: 0.2
        """

    @abstractmethod
    def display_subprocess_output(self, stream_name, line):
        """
        Method called for each line of stdout/stderr as obtained from a
        subprocess.

        .. versionadded:: 0.2
        """


class ITestInstallerObserver(IShellCommandObserver):
    """
    Test installer observer class.

    Allows the caller to observe interesting actions that occur during
    installation process. It is used by the command line UI.

        .. versionadded:: 0.2
    """

    @abstractmethod
    def about_to_install_packages(self, package_list):
        """
        Method called when a list of packages is about to be installed by the
        installer

        .. versionadded:: 0.2
        """

    @abstractmethod
    def did_install_packages(self, package_list):
        """
        Method called when a package has been installed by the installer

        .. versionadded:: 0.2
        """

    @abstractmethod
    def about_to_download_file(self, url):
        """
        Method called when a file is about to be downloaded

        .. versionadded:: 0.2
        """

    @abstractmethod
    def did_download_file(self, url):
        """
        Method called when a file has been downloaded

        .. versionadded:: 0.2
        """


class ITestRunnerObserver(IShellCommandObserver):
    """
    Test runner observer class.

    Allows the caller to observe interesting actions that occur during testing
    process. It is used by the command line UI.

        .. versionadded:: 0.2
    """



# Copyright (c) 2010 Linaro Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import abrek.testdef

spandex_benchmark = "/usr/share/spandex/bd/benchmarks/opengles2/benchmark_smoke.txt"

INSTALL_STEPS = [
    'sudo add-apt-repository ppa:linaro-maintainers/user-platforms',
    'sudo apt-get update',
    'sudo apt-get install -y --force-yes spandex-gles2 spandex-benchmarks-gles2'
    ]

RUNSTEPS = [
    "spandex-gles2 %s > /dev/null" % spandex_benchmark,
    "python /usr/share/spandex/bd/python/wikireport.py result.txt"
    ]

PATTERN = "^\|\s+(?P<test_case_id>[^|]+?)\s+\|\s+(?P<measurement>\d+(\.\d+)?)"

inst = abrek.testdef.AbrekTestInstaller(INSTALL_STEPS,
                                        deps=["python-software-properties"])
run = abrek.testdef.AbrekTestRunner(RUNSTEPS)
parse = abrek.testdef.AbrekTestParser(PATTERN,
                                      appendall={'units':'reps/s',
                                                 'result':'pass'})

testobj = abrek.testdef.AbrekTest(testname="spandex-gles2", installer=inst,
                                  runner=run, parser=parse)



# Copyright (c) 2010, 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import absolute_import

import base64
import datetime
import logging
import os
import uuid

from linaro_dashboard_bundle.io import DocumentIO

from lava_test.core import hwprofile, swprofile
from lava_test.utils import merge_dict, mkdir_p


class TestArtifacts(object):
    """
    Class representing test run artifacts, that is, static leftovers
    independent of the wrapper class that encapsulates test handling.

    .. versionadded:: 0.2
    """

    def __init__(self, test_id, result_id, config):
        self._test_id = test_id
        self._result_id = result_id
        self._config = config
        self._bundle = None

    @classmethod
    def allocate(cls, test_id, config):
        """
        Allocate new test artifacts object that corresponds to the specified
        test_id. This constructs a new result_id and creates the corresponding
        filesystem directory that holds those artifacts.

        .. versionadded:: 0.2
        """
        result_id = (
            "{test_id}.{time.tm_year:04}-{time.tm_mon:02}-{time.tm_mday:02}T"
            "{time.tm_hour:02}:{time.tm_min:02}:{time.tm_sec:02}Z").format(
                test_id=test_id,
                time=datetime.datetime.utcnow().timetuple())
        self = cls(test_id, result_id, config)
        logging.debug("Creating result directory: %r", self.results_dir)
        mkdir_p(self.results_dir)
        return self

    @property
    def test_id(self):
        """
        The ID of the test this run is associated with

        .. versionadded:: 0.2
        """
        return self._test_id

    @property
    def result_id(self):
        """
        The ID of the test run.

        This field is different from analyzer_assigned_uuid at this time but
        may change in the future. The purpose of this field is to identify the
        test run and be able to locate attachments/log files/bundle on the file
        system.

        .. versionadded:: 0.2
        """
        return self._result_id

    @property
    def results_dir(self):
        """
        Pathname of a directory with test run artifacts (log files, crash
        dumps, etc).

        .. versionadded:: 0.2
        """
        return os.path.join(self._config.resultsdir, self.result_id)

    def load_bundle(self):
        """
        Load the results bundle from disk.

        The bundle is also validated if linaro-dashboard-bundle library is
        installed.
        """
        with open(self.bundle_pathname, 'rt') as stream:
            self._bundle = DocumentIO.load(stream)[1]

    def dumps_bundle(self):
        return DocumentIO.dumps(self._bundle)

    def save_bundle(self):
        """
        Save the results bundle to the disk

        The bundle is also validated if linaro-dashboard-bundle library is
        installed.
        """
        self.save_bundle_as(self.bundle_pathname)

    def save_bundle_as(self, pathname):
        """
        Save the results bundle to the specified file on disk.

        The bundle should have been created or loaded earlier
        """
        with open(pathname, 'wt') as stream:
            DocumentIO.dump(stream, self._bundle)

    @property
    def bundle(self):
        """
        The deserialized bundle object.

        This can be either created with create_bundle() or loaded
        from disk with load_bundle()
        """
        return self._bundle

    def create_initial_bundle(self,
                      skip_software_context=False,
                      skip_hardware_context=False,
                      time_check_performed=False,
                      analyzer_assigned_uuid=None):
        """
        Create the bundle object.

        This creates a typical bundle structure. Optionally it can also add
        software and hardware context information.

        For a complete bundle you may want to add attachments and incorporate
        parse results by calling appropriate methods after loading or creating
        the initial bundle.
        """
        TIMEFORMAT = '%Y-%m-%dT%H:%M:%SZ'
        # Generate UUID and analyzer_assigned_date for the test run
        if analyzer_assigned_uuid is None:
            analyzer_assigned_uuid = str(uuid.uuid1())
        analyzer_assigned_date = datetime.datetime.utcnow()
        # Create basic test run structure
        test_run = {
            'test_id': self.test_id,
            'analyzer_assigned_date': analyzer_assigned_date.strftime(
                TIMEFORMAT),
            'analyzer_assigned_uuid': analyzer_assigned_uuid,
            'time_check_performed': time_check_performed,
            "test_results": [],
            "attachments": [],
        }
        # Store hardware and software context if requested
        if not skip_software_context:
            test_run['software_context'] = swprofile.get_software_context()
        if not skip_hardware_context:
            test_run['hardware_context'] = hwprofile.get_hardware_context()
        # Create the bundle object
        self._bundle = {
            'format': 'Dashboard Bundle Format 1.2',
            'test_runs': [test_run]}

    @property
    def test_run(self):
        try:
            return self._bundle["test_runs"][0]
        except KeyError:
            raise AttributeError("test_run can be accessed only after you load"
                                 " or create an initial bundle")

    def attach_file(self, real_pathname, stored_pathname, mime_type):
        """
        Append an attachment to the test run.

        The file is only attached if real_pathname designates an existing,
        nonempty file. If the mime_type starts with 'text/' the file is opened
        in text mode, otherwise binary mode is used.
        """
        if not os.path.exists(real_pathname):
            return
        if mime_type.startswith('text/'):
            mode = 'rt'
        else:
            mode = 'rb'
        with open(real_pathname, mode) as stream:
            data = stream.read()
        if not data:
            return
        self.test_run['attachments'].append({
            "pathname": stored_pathname,
            "mime_type": mime_type,
            "content": base64.standard_b64encode(data)})

    def incorporate_parse_results(self, parse_results):
        """
        Merge the data returned by the test parser into the current test run.

        Non-overlapping data is simply added. Overlapping data is either merged
        (lists are extended, dictionaries are recursively merged) or
        overwritten (all other types).
        """
        assert isinstance(parse_results, dict)
        # Use whatever the parser gave us to improve the results
        logging.debug("Using parser data to enrich test run details")
        merge_dict(self.test_run, parse_results)

    def attach_standard_files_to_bundle(self):
        """
        Attach standard output and standard error log files to the bundle.

        Both files are only attached if exist and non-empty. The attachments
        are actually associated with a test run, not a bundle, but the
        description is good enough for simplicity.
        """
        self.attach_file(self.stdout_pathname, "testoutput.log", "text/plain")
        self.attach_file(self.stderr_pathname, "testoutput.err", "text/plain")

    @property
    def bundle_pathname(self):
        """
        Pathname of the result bundle.

        The bundle contains the snapshot of environment information as well as
        test identity and is created when you invoke ITest.run().

        The bundle file name is always "testdata.json"

        .. versionadded:: 0.2
        """
        return self.get_artefact_pathname("testdata.json")

    @property
    def stdout_pathname(self):
        """
        Pathname of the log file of the standard output as returned by the test
        program.

        The log file name is always "testoutput.log"

        .. versionadded:: 0.2
        """
        return self.get_artefact_pathname("testoutput.log")

    @property
    def stderr_pathname(self):
        """
        Pathname of the log file of the standard output as returned by the test
        program.

        The log file name is always "testoutput.err"

        .. versionadded:: 0.2
        """
        return self.get_artefact_pathname("testoutput.err")

    def get_artefact_pathname(self, artefact_name):
        """
        Return a pathname of a test run artefact file.

        This is more useful than hard-coding the path as it allows the test
        runner not to worry about the location of the results directory.

        .. versionadded:: 0.2
        """
        return os.path.join(self.results_dir, artefact_name)



# Copyright (c) 2010, 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import json


class LavaTestConfig(object):

    def __init__(self):
        home = os.environ.get('HOME', '/')
        baseconfig = os.environ.get('XDG_CONFIG_HOME',
                     os.path.join(home, '.config'))
        basedata = os.environ.get('XDG_DATA_HOME',
                     os.path.join(home, '.local', 'share'))
        self.configdir = os.path.join(baseconfig, 'lava_test')
        self.installdir = os.path.join(
            basedata, 'lava_test', 'installed-tests')
        self.resultsdir = os.path.join(basedata, 'lava_test', 'results')
        self.registry = self._load_registry()

    @property
    def _registry_pathname(self):
        return os.path.join(self.configdir, "registry.json")

    def _load_registry(self):
        try:
            with open(self._registry_pathname) as stream:
                return json.load(stream)
        except (IOError, ValueError):
            return self._get_default_registry()

    def _save_registry(self):
        if not os.path.exists(self.configdir):
            os.makedirs(self.configdir)
        with open(self._registry_pathname, "wt") as stream:
            json.dump(self.registry, stream, indent=2)

    def _get_default_registry(self):
        return {
            "format": "Lava-Test Test Registry 1.0",
            "providers": [{
                "entry_point": "lava_test.core.providers:BuiltInProvider"
            }, {
                "entry_point": "lava_test.core.providers:PkgResourcesProvider",
                "config": {"namespace": "lava_test.test_definitions"}
            },
            {
                "entry_point": "lava_test.core.providers:RegistryProvider",
                "config": {"entries": []}
            }]
        }

    def get_provider_config(self, entry_point_name):
        if "providers" not in self.registry:
            self.registry["providers"] = []
        for provider_info in self.registry["providers"]:
            if provider_info.get("entry_point") == entry_point_name:
                break
        else:
            provider_info = {"entry_point": entry_point_name}
            self.registry["providers"].append(provider_info)
        if "config" not in provider_info:
            provider_info["config"] = {}
        return provider_info["config"]

_config = None


def get_config():
    global _config
    if _config is not None:
        return _config
    return LavaTestConfig()


def set_config(config):
    global _config
    _config = config



# Copyright (c) 2010, 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


"""
:mod:`lava_test.api.delegates` -- Interface classes for test delegates
======================================================================

.. module: lava_test.api.delegates

    :synopsis: Interface classes for test delegates
"""

from abc import abstractmethod, abstractproperty

from lava_test.api import _Interface


class ITestInstaller(_Interface):
    """
    Test installer delegate class.

    Wraps the knowledge on how to install a test. It is most helpful with
    :class:`~lava_test.core.tests.Test` that delegates actual actions to helper
    classes.

    .. versionadded:: 0.2
    """

    @abstractmethod
    def install(self, observer):
        """
        Install the test program.

        :param observer:
            Observer object that makes it possible to monitor the actions
            performed by the test installer.
        :type observer:
            :class:`~lava_test.api.observers.ITestInstallerObserver`

        .. versionadded:: 0.2
        """


class ITestRunner(_Interface):
    """
    Test runner delegate.

    Wraps the knowledge on how to run a test. It is most helpful with
    :class:`lava_test.core.tests.Test` that delegates actual actions to
    helper classes.

    .. versionadded:: 0.2
    """

    @abstractmethod
    def run(self, artifacts, observer, test_options):
        """
        Run the test and create artifacts (typically log files).

        Artifacts must be created in the directory specified by various methods
        and properties of of :class:`lava_test.core.TestArtifacts`.

        :param artifacts:
            Object that describes where to store test run artifacts
        :type artifacts: :class:`~lava_test.core.artifacts.TestArtifacts`.
        :param observer:
            Observer object that makes it possible to monitor the actions
            performed by the test runner.
        :type observer: :class:`~lava_test.api.observers.ITestRunnerObserver`
        :param test_options:
           A string with space separated options to pass to the test.

        :return true if any test step return none-zero return code
        .. versionadded:: 0.2
        """


class ITestParser(_Interface):
    """
    Test artefact parser delegate.

    Wraps the knowledge on how to parse the artifacts of a previous test run.
    It is most helpful with :class:`~lava_test.core.tests.Test` that delegates
    actual actions to helper classes.

        .. versionadded:: 0.2
    """

    @abstractmethod
    def parse(self, artifacts):
        """
        Parse the artifacts of a previous test run and return a dictionary with
        a partial TestRun object.

        :param artifacts:
            Object that describes where to find test run artifacts
        :type artifacts: :class:`~lava_test.core.artifacts.TestArtifacts`.

        .. versionadded:: 0.2
        """

    @abstractproperty
    def results(self):
        """
        Results dictionary to be merged with TestRun object inside the bundle.

        .. seealso::
            :meth:`~lava_test.core.artifacts.TestArtifacts.incorporate_parse_results`

        .. versionadded:: 0.1
        """



#, Copyright (c) 2010, 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
The Linux Test Project is a collection of tools for testing Linux with a focus
on the kernel.

**URL:** http://ltp.sourceforge.net

**Default options:** -f syscalls -p -q
"""

import re


from lava_test.core.installers import TestInstaller
from lava_test.core.parsers import TestParser
from lava_test.core.runners import TestRunner
from lava_test.core.tests import Test


VERSION = "20100831"
URL = ('http://downloads.sourceforge.net/project/ltp/'
       'LTP Source/ltp-%s/ltp-full-%s.bz2') % (VERSION, VERSION)
MD5 = "6982c72429a62f3917c13b2d529ad1ce"
DEPS = ['bzip2', 'flex', 'bison', 'make', 'build-essential']

SCRIPT = """
tar -xjf ltp-full-20100831.bz2
mkdir build
cd ltp-full-20100831
./configure --prefix=$(readlink -f ../build)
make all
SKIP_IDCHECK=1 make install
"""
DEFAULT_OPTIONS = "-f syscalls -p -q"

INSTALLSTEPS = ["echo '%s' > installltp.sh" % SCRIPT,
                'chmod +x installltp.sh',
                './installltp.sh']
RUNSTEPS = ['cd build && sudo ./runltp $(OPTIONS)']
PATTERN = (
    "^(?P<test_case_id>\S+)"
    "    (?P<subid>\d+)"
    "  (?P<result>\w+)"
    "  :  (?P<message>.+)")
FIXUPS = {
    "TBROK": "fail",
    "TCONF": "skip",
    "TFAIL": "fail",
    "TINFO": "unknown",
    "TPASS": "pass",
    "TWARN": "unknown"}


class LTPParser(TestParser):

    def parse(self, artifacts):
        filename = artifacts.stdout_pathname
        pat = re.compile(self.pattern)
        with open(filename, 'r') as fd:
            for line in fd.readlines():
                match = pat.search(line)
                if match:
                    results = match.groupdict()
                    subid = results.pop('subid')
                    #The .0 results in ltp are all TINFO, filtering them
                    #should help eliminate meaningless, duplicate results
                    if subid == '0':
                        continue
                    results['test_case_id'] += "." + subid
                    self.results['test_results'].append(
                        self.analyze_test_result(results))


installer = TestInstaller(INSTALLSTEPS, deps=DEPS, url=URL, md5=MD5)
runner = TestRunner(RUNSTEPS, default_options=DEFAULT_OPTIONS)
parser = LTPParser(PATTERN, fixupdict=FIXUPS)

testobj = Test(
    test_id="ltp",
    test_version=VERSION,
    installer=installer,
    runner=runner,
    parser=parser)



# Copyright (c) 2010, 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
The POSIX Test Suite is an open source test suite with the goal of
performing conformance, functional, and stress testing of the IEEE
1003.1-2001 System Interfaces specification However the automation here
does not support the stress test runs.

**URL:** http://sourceforge.net/projects/posixtest/

**Default options:** None
"""

import re

from lava_test.core.installers import TestInstaller
from lava_test.core.parsers import TestParser
from lava_test.core.runners import TestRunner
from lava_test.core.tests import Test


VERSION = "20100831"
URL = ("http://downloads.sourceforge.net/project/ltp/LTP Source/ltp-%s/"
      "ltp-full-%s.bz2") % (VERSION, VERSION)
MD5 = "6982c72429a62f3917c13b2d529ad1ce"
DEFAULT_OPTIONS = ""
INSTALLSTEPS = ['tar -xjf ltp-full-20100831.bz2']
DEPS = ['gcc', 'bzip2']
RUNSTEPS = [
    ('cd ltp-full-20100831/testcases/open_posix_testsuite/'
     ' && make $(OPTIONS)')]

PATTERN = ("((?P<test_case_id>\A(\w+[/]+)+\w+[-]*\w*[-]*\w*)"
           " .*? (?P<result>\w+))")
FIXUPS = {
    "FAILED": "fail",
    "INTERRUPTED": "skip",
    "PASSED": "pass",
    "UNRESOLVED": "unknown",
    "UNSUPPORTED": "skip",
    "UNTESTED": "skip",
    "SKIPPING": "skip"
}


class PosixParser(TestParser):

    def parse(self, artifacts):
        filename = "testoutput.log"
        pat = re.compile(self.pattern)
        with open(filename) as fd:
            for lineno, line in enumerate(fd, 1):
                match = pat.match(line)
                if not match:
                    continue
                results = match.groupdict()
                test_case_id = results['test_case_id']
                results['test_case_id'] = test_case_id.replace("/", ".")
                results["log_filename"] = "testoutput.log"
                results["log_lineno"] = lineno
                self.results['test_results'].append(
                    self.analyze_test_result(results))


installer = TestInstaller(INSTALLSTEPS, deps=DEPS, url=URL, md5=MD5)
runner = TestRunner(RUNSTEPS, default_options=DEFAULT_OPTIONS)
parser = PosixParser(PATTERN, fixupdict=FIXUPS)

testobj = Test(
    test_id="posixtestsuite",
    test_version=VERSION,
    installer=installer,
    runner=runner,
    parser=parser)



# Copyright (c) 2010, 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Simple test for smem that simply reports the memory usage with shared memory divided proportionally

**URL:** None

**Default options:** None
"""


from lava_test.core.installers import TestInstaller
from lava_test.core.parsers import TestParser
from lava_test.core.runners import TestRunner
from lava_test.core.tests import Test

DEFAULT_OPTIONS = ""
RUNSTEPS = ['smem -w $(OPTIONS) | tail -n 3']
PATTERN = "^(?P<test_case_id>(\w+\s)+)\s\s+(?P<measurement>\d+)"
DEPS = ['smem']

installer = TestInstaller(deps=DEPS)
runner = TestRunner(RUNSTEPS, default_options=DEFAULT_OPTIONS)
parser = TestParser(PATTERN, appendall={'units': 'KB', 'result': 'pass'})

testobj = Test(
    test_id="smem",
    installer=installer,
    runner=runner,
    parser=parser)



#!/usr/bin/env python

# Copyright (c) 2010, 2011, 2012 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from setuptools import setup, find_packages


setup(
    name='lava-test',
    version=":versiontools:lava_test:",
    author='Linaro Validation Team',
    author_email='linaro-dev@lists.linaro.org',
    url='https://launchpad.net/lava-test',
    description='LAVA test execution framework',
    long_description=open("README").read(),
    packages=find_packages(exclude=['tests']),
    license="GNU GPLv3",
    test_suite='unittest2.collector',
    entry_points="""
    [console_scripts]
    lava-test=lava_test.main:main
    [lava_test.commands]
    version=lava_test.commands:version
    list-tests=lava_test.commands:list_tests
    list-installed=lava_test.commands:list_installed
    install=lava_test.commands:install
    uninstall=lava_test.commands:uninstall
    run=lava_test.commands:run
    parse=lava_test.commands:parse
    show=lava_test.commands:show
    register_test=lava_test.commands:register_test
    unregister_test=lava_test.commands:unregister_test
    reset=lava_test.commands:reset
    """,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.6",
        "Topic :: Software Development :: Testing",
    ],
    install_requires=[
        'lava-tool >= 0.2',
        'versiontools >= 1.4',
        'linaro_dashboard_bundle',
    ],
    setup_requires=[
        'versiontools >= 1.4'
    ],
    tests_require=[
        'unittest2',
        'mocker >= 1.1',
    ],
    zip_safe=False,
    include_package_data=True)



# Copyright (c) 2010, 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from lava_test.utils import read_file


def get_packages(apt_cache=None):
    """ Get information about the packages installed

    apt_cache - if not provided, this will be read from the system
    """
    import apt
    if apt_cache == None:
        apt_cache = apt.Cache()
    packages = []
    for apt_pkg in apt_cache:
        if hasattr(apt_pkg, 'is_installed'):
            is_installed = apt_pkg.is_installed
        else:
            is_installed = apt_pkg.isInstalled  # old style API
        if is_installed:
            pkg = {
                "name": apt_pkg.name,
                "version": apt_pkg.installed.version}
            packages.append(pkg)
    return packages


def get_software_context(apt_cache=None, lsb_information=None):
    """ Return dict used for storing software_context information

    test_id - Unique identifier for this test
    time_check - whether or not a check was performed to see if
            the time on the system was synced with a time server
    apt_cache - if not provided, this will be read from the system
    lsb_information - if not provided, this will be read from the system
    """
    software_context = {}
    software_context['image'] = get_image(lsb_information)
    software_context['packages'] = get_packages(apt_cache)
    return software_context


def get_image(lsb_information=None):
    """ Get information about the image we are running

    If /etc/buildstamp exists, get the image id from that.  Otherwise
    just use the lsb-release description for a rough idea.
    """
    try:
        buildstamp = read_file("/etc/buildstamp")
        name = buildstamp.splitlines()[1]
    except IOError:
        import lsb_release

        if lsb_information == None:
            lsb_information = lsb_release.get_distro_information()
        name = lsb_information['DESCRIPTION']
    return {"name": name}



# Copyright (c) 2010, 2011, 2012 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import logging

import os
import shutil
import subprocess

from lava_tool.interface import Command as LavaCommand
from lava_tool.interface import LavaCommandError
import versiontools

from lava_test.api.observers import (
    ITestInstallerObserver,
    ITestRunnerObserver)
from lava_test.core.artifacts import TestArtifacts
from lava_test.core.config import get_config
from lava_test.core.loader import TestLoader
from lava_test.utils import Cache

class Command(LavaCommand, ITestInstallerObserver, ITestRunnerObserver):

    def __init__(self, parser, args):
        super(Command, self).__init__(parser, args)
        self._config = get_config()
        self._test_loader = TestLoader(self._config)

        if self.args.verbose:
            logging.root.setLevel(logging.DEBUG)

    @classmethod
    def register_arguments(cls, parser):
        parser.add_argument(
           "-v", "--verbose",
           action="store_true",
           default=False,
           help="Be verbose about undertaken actions")
        parser.add_argument(
            "-q", "--quiet",
            action="store_true",
            default=False,
            help="Be less verbose about undertaken actions")
        parser.add_argument(
            "-Q", "--quiet-subcommands",
            action="store_true",
            default=False,
            help="Hide the output of all sub-commands (including tests)")

    def say(self, text, *args, **kwargs):
        print "LAVA:", text.format(*args, **kwargs)

    def about_to_install_packages(self, package_list):
        if self.args.quiet:
            return
        self.say("Installing packages: {0}", ", ".join(package_list))

    def about_to_run_shell_command(self, cmd):
        if self.args.quiet:
            return
        self.say("Running shell command: {0!r}", cmd)

    def about_to_download_file(self, url):
        if self.args.quiet:
            return
        self.say("Downloading file from: {0!r}", url)

    def did_install_packages(self, package_list):
        pass

    def did_run_shell_command(self, cmd, returncode):
        if returncode is None:
            self.say("Command {0!r} was terminated prematurely", cmd)
        elif returncode != 0:
            self.say("Command {0!r} returned non-zero exit status {1}",
                     cmd, returncode)

    def did_download_file(self, url):
        pass

    def display_subprocess_output(self, stream_name, line):
        if self.args.quiet_subcommands:
            return
        if stream_name == 'stdout':
            self.say('(stdout) {0}', line.rstrip())
        elif stream_name == 'stderr':
            self.say('(stderr) {0}', line.rstrip())


class list_tests(Command):
    """
    List available tests

    .. program:: lava-test list-tests

    Lists all available tests, grouping them by provider.
    """

    def invoke(self):
        for provider in self._test_loader.get_providers():
            test_list = [provider[test_id] for test_id in provider]
            if not test_list:
                continue
            self.say("{0}", provider.description)
            for test in test_list:
                self.say(" - {test_id}", test_id=test.test_id)


class list_installed(Command):
    """
    List installed tests
    """

    def invoke(self):
        for provider in self._test_loader.get_providers():
            test_list = [provider[test_id] for test_id in provider]
            if not test_list:
                continue
            self.say("{0}", provider.description)
            count = 0
            for test in test_list:
                if not test.is_installed:
                    continue
                self.say(" - {test_id}", test_id=test.test_id)
                count += 1
            if not count:
                self.say("No tests installed")


class TestAffectingCommand(Command):

    INSTALL_REQUIRED = False

    @classmethod
    def register_arguments(cls, parser):
        super(TestAffectingCommand, cls).register_arguments(parser)
        parser.add_argument("test_id",
                            help="Test or test suite identifier")

    def invoke(self):
        try:
            test = self._test_loader[self.args.test_id]
        except KeyError:
            try:
                test = self._test_loader[self.args.test_id.replace('-', '_')]
            except KeyError:
                raise LavaCommandError("There is no test with the specified ID")
        return self.invoke_with_test(test)


class install(TestAffectingCommand):
    """
    Install a test program
    """

    def invoke_with_test(self, test):
        if test.is_installed:
            raise LavaCommandError("This test is already installed")
        try:
            test.install(self)
        except (subprocess.CalledProcessError, RuntimeError) as ex:
            raise LavaCommandError(str(ex))


class uninstall(TestAffectingCommand):
    """
    Uninstall a test program
    """

    def invoke_with_test(self, test):
        if not test.is_installed:
            raise LavaCommandError("This test is not installed")
        test.uninstall()


class run(TestAffectingCommand):
    """
    Run a previously installed test program
    """

    @classmethod
    def register_arguments(cls, parser):
        super(run, cls).register_arguments(parser)
        group = parser.add_argument_group("initial bundle configuration")
        group.add_argument(
            "-S", "--skip-software-context",
            default=False,
            action="store_true",
            help=("Do not store the software context in the"
                  " initial bundle. Typically this saves OS"
                  " image name and all the installed software"
                  " packages."))
        group.add_argument(
            "-H", "--skip-hardware-context",
            default=False,
            action="store_true",
            help=("Do not store the hardware context in the"
                  " initial bundle. Typically this saves CPU,"
                  " memory and USB device information."))
        group.add_argument(
            "--trusted-time",
            default=False,
            action="store_true",
            help=("Indicate that the real time clock has"
                  " accurate data. This can differentiate"
                  " test results created on embedded devices"
                  " that often have inaccurate real time"
                  " clock settings."))
        group.add_argument(
            "--analyzer-assigned-uuid",
            default=None,
            metavar="UUID",
            help=("Set the analyzer_assigned_uuid to the specified value."
                  " This will prevent the test device from attempting"
                  " to generate an UUID by itself. This option may be"
                  " required if the test device has unreliable real"
                  " time clock (no battery backed, not ensure to be"
                  " up-to-date) and unreliable/random hardware ethernet "
                  " address."))

        group = parser.add_argument_group("complete bundle configuration")
        group.add_argument(
            "-o", "--output",
            default=None,
            metavar="FILE",
            help=("After running the test parse the result"
                  " artifacts, fuse them with the initial"
                  " bundle and finally save the complete bundle"
                  " to the  specified FILE."))
        group.add_argument(
            "-A", "--skip-attachments",
            default=False,
            action="store_true",
            help=("Do not store standard output and standard"
                  " error log files as attachments. This"
                  " option is only affecting the bundle"
                  " created with --output, the initial bundle"
                  " is not affected as it never stores any"
                  " attachments."))

        parser.add_argument(
            "-t", "--test-options",
            default=None,
            help=(
                "Override the default test options."
                " The value is passed verbatim to test definition. Typically"
                " this is simply used in shell commands by expanding the"
                " string $(OPTIONS). Please refer to the built-in"
                " peacekeeper.py for examples. Depending on your shell you"
                " probably have to escape spaces and other special"
                " characters if you wish to include them in your argument"
                " options."))

    def invoke_with_test(self, test):
        # Validate analyzer_assigned_uuid
        if self.args.analyzer_assigned_uuid:
            import uuid
            try:
                self.analyzer_assigned_uuid = str(
                    uuid.UUID(self.args.analyzer_assigned_uuid))
            except ValueError as exc:
                self.parser.error("--analyzer-assigned-uuid: %s" % exc)
        if not test.is_installed:
            raise LavaCommandError("The specified test is not installed")
        try:
            artifacts, run_fail = test.run(
                self, test_options=self.args.test_options)
        except subprocess.CalledProcessError as ex:
            if ex.returncode is None:
                raise LavaCommandError("Command %r was aborted" % ex.cmd)
            else:
                raise LavaCommandError(str(ex))
        except RuntimeError as ex:
            raise LavaCommandError(str(ex))
        self.say("run complete, result_id is {0!r}", artifacts.result_id)
        try:
            artifacts.create_initial_bundle(
                self.args.skip_software_context,
                self.args.skip_hardware_context,
                self.args.trusted_time,
                self.args.analyzer_assigned_uuid)
        except ImportError as exc:
            msg_template = (
                "Unable to probe for software context. Install the '%s'"
                " package or invoke lava-test run with"
                " '--skip-software-context'")
            if exc.message == "No module named apt":
                raise LavaCommandError(msg_template % "python-apt")
            elif exc.message == "No module named lsb_release":
                raise LavaCommandError(msg_template % "lsb-release")
            else:
                raise
        artifacts.save_bundle()
        if self.args.output:
            parse_results = test.parse(artifacts)
            artifacts.incorporate_parse_results(parse_results)
            if not self.args.skip_attachments:
                artifacts.attach_standard_files_to_bundle()
            artifacts.save_bundle_as(self.args.output)
        if run_fail:
            raise LavaCommandError(
                'Some of test steps returned non-zero exit code')


class parse(TestAffectingCommand):
    """
    Parse the results of previous test run
    """

    @classmethod
    def register_arguments(cls, parser):
        super(parse, cls).register_arguments(parser)
        parser.add_argument("result_id",
                            help="Test run result identifier")
        group = parser.add_argument_group("complete bundle configuration")
        group.add_argument("-o", "--output",
                            default=None,
                            metavar="FILE",
                           help=("After running the test parse the result"
                                 " artifacts, fuse them with the initial"
                                 " bundle and finally save the complete bundle"
                                 " to the  specified FILE."))
        group.add_argument("-A", "--skip-attachments",
                            default=False,
                            action="store_true",
                            help=("Do not store standard output and standard"
                                  " error log files as attachments. This"
                                  " option is only affecting the bundle"
                                  " created with --output, the initial bundle"
                                  " is not affected as it never stores any"
                                  " attachments."))

    def invoke_with_test(self, test):
        artifacts = TestArtifacts(
            self.args.test_id, self.args.result_id, self._config)
        if not os.path.exists(artifacts.bundle_pathname):
            raise LavaCommandError("Specified result does not exist")
        artifacts.load_bundle()
        parse_results = test.parse(artifacts)
        artifacts.incorporate_parse_results(parse_results)
        self.say("Parsed {0} test results",
                 len(artifacts.bundle["test_runs"][0]["test_results"]))
        logging.debug(artifacts.dumps_bundle())
        if self.args.output:
            if not self.args.skip_attachments:
                artifacts.attach_standard_files_to_bundle()
            artifacts.save_bundle_as(self.args.output)


class show(Command):
    """
    Display the output from a previous test run
    """

    @classmethod
    def register_arguments(cls, parser):
        super(show, cls).register_arguments(parser)
        parser.add_argument("result_id",
                            help="Test run result identifier")

    def invoke(self):
        artifacts = TestArtifacts(None, self.args.result_id, self._config)
        if not os.path.exists(artifacts.results_dir):
            raise LavaCommandError("Specified result does not exist")
        if os.path.exists(artifacts.stdout_pathname):
            with open(artifacts.stdout_pathname, "rt") as stream:
                for line in iter(stream.readline, ''):
                    self.display_subprocess_output("stdout", line)
        if os.path.exists(artifacts.stderr_pathname):
            with open(artifacts.stderr_pathname, "rt") as stream:
                for line in iter(stream.readline, ''):
                    self.display_subprocess_output("stderr", line)


class version(Command):
    """
    Show LAVA Test version
    """

    def invoke(self):
        self.say("version details:")
        for framework in self._get_frameworks():
            self.say(" - {framework}: {version}",
                     framework=framework.__name__,
                     version=versiontools.format_version(
                         framework.__version__, framework))

    def _get_frameworks(self):
        import lava_tool
        import lava_test
        import linaro_dashboard_bundle
        import linaro_json
        return [
            lava_test,
            lava_tool,
            linaro_dashboard_bundle,
            linaro_json]


class register_test(Command):
    """
    Register remote test
    """

    @classmethod
    def register_arguments(cls, parser):
        super(register_test, cls).register_arguments(parser)
        parser.add_argument("test_url",
                            help="Url for test definition file")

    def invoke(self):
        try:
            from lava_test.core.providers import RegistryProvider
            RegistryProvider.register_remote_test(self.args.test_url)
        except ValueError as exc:
            raise LavaCommandError("Unable to register test: %s" % exc)
        except KeyError:
            raise LavaCommandError("There is no test_url")


class unregister_test(Command):
    """
    Remove a declarative test from the registry
    This command does the reverse of lava-test register. You need to pass the
    same URL you've used in `lava-test register-test`
    """

    @classmethod
    def register_arguments(cls, parser):
        super(unregister_test, cls).register_arguments(parser)
        parser.add_argument(
            "url",
            metavar="URL",
            help="URL of the test definition file")

    def invoke(self):
        from lava_test.core.providers import RegistryProvider
        try:
            RegistryProvider.unregister_remote_test(self.args.url)
        except ValueError:
            raise LavaCommandError("This test is not registered")


class reset(Command):
    """
    Reset the lava-test environment by removing
    cached items, all registerered simple declactive tests and configuration files
    """

    def invoke(self):
        print "Delete " + self._config.configdir
        shutil.rmtree(self._config.configdir, ignore_errors=True)
        print "Delete " + self._config.installdir
        shutil.rmtree(self._config.installdir, ignore_errors=True)
        print "Delete " + self._config.resultsdir
        shutil.rmtree(self._config.resultsdir, ignore_errors=True)
        cache = Cache.get_instance()
        print "Delete " + cache.cache_dir
        shutil.rmtree(cache.cache_dir, ignore_errors=True)



# Copyright (c) 2012 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
A program that performs a series of tests to benchmark the XRender X11
extension.  It also performs the same tests using the Imlib2 library to
provide a base for comparing performance.

**URL:** None

**Default options:** None
"""


import re
import abrek.testdef

class RenderBenchParser(abrek.testdef.AbrekTestParser):
    def parse(self):
        PAT1 = "^Test: (?P<test_case_id>.*)"
        PAT2 = "^Time: (?P<measurement>\d+\.\d+)"
        filename = "testoutput.log"
        pat1 = re.compile(PAT1)
        pat2 = re.compile(PAT2)
        cur_test = None
        with open(filename, 'r') as fd:
            for line in fd:
                match = pat1.search(line)
                if match:
                    cur_test = match.groupdict()['test_case_id']
                else:
                    match = pat2.search(line)
                    if match:
                        d = match.groupdict()
                        d['test_case_id'] = cur_test
                        self.results['test_results'].append(d)

        self.appendtoall({'units':'seconds', 'result':'pass'})

RUNSTEPS = ["render_bench"]

inst = abrek.testdef.AbrekTestInstaller(deps=["render-bench"])
run = abrek.testdef.AbrekTestRunner(RUNSTEPS)
parse = RenderBenchParser()

testobj = abrek.testdef.AbrekTest(testname="render-bench", installer=inst,
                                  runner=run, parser=parse)
